package com.winhc.repal.controller;

import com.winhc.common.base.BeanResponse;
import com.winhc.common.enums.CodeMsg;
import com.winhc.repal.annotation.RolePermission;
import com.winhc.repal.enums.PermissionEnum;
import com.winhc.repal.model.dto.AddRepalBookDTO;
import com.winhc.repal.model.dto.DeleteRepalBookDTO;
import com.winhc.repal.model.dto.HomeTypeDTO;
import com.winhc.repal.model.dto.RepalBookNameDTO;
import com.winhc.repal.model.vo.HomeGradeVO;
import com.winhc.repal.model.vo.HomeReceivableVO;
import com.winhc.repal.model.vo.RepalBookInfoVO;
import com.winhc.repal.model.vo.ReturnedMoneyVO;
import com.winhc.repal.service.RepalBookService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * @description AccountBook Controller
 * @author Generator
 * @date 2020-12-15
 */
@Slf4j
@Api(tags = "账本")
@RestController
@RequestMapping("/repalBook")
public class RepalBookController {



    @Autowired
    private RepalBookService repalBookService;


    @ApiOperation("首页应收/应付部分")
    @RequestMapping(value = "/home/receivable/{billType}", method = RequestMethod.POST)
    public BeanResponse<HomeReceivableVO> getHomeReceivable(@PathVariable("billType")Integer billType, HomeTypeDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBookService.getHomeReceivable(billType, dto));
    }


    @ApiOperation("首页账本应收账款动态")
    @RequestMapping(value = "/dynamic", method = RequestMethod.GET)
    public BeanResponse<List<List<Integer>>> getRepalBookDynamic(HomeTypeDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBookService.getRepalBookDynamic(dto));
    }

    @ApiOperation("首页账本账款评级数据")
    @RequestMapping(value = "/grade", method = RequestMethod.GET)
    public BeanResponse<List<HomeGradeVO>> getGradeRate(HomeTypeDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBookService.getGradeRateList(dto));
    }

    @ApiOperation("首页账本回款统计折线图")
    @RequestMapping(value = "/graph", method = RequestMethod.GET)
    public BeanResponse<List<ReturnedMoneyVO>> getGraph(HomeTypeDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBookService.getGraph(dto));
    }

    @ApiOperation("删除账本")
    @RequestMapping(value = "/deleteRepalBook", method = RequestMethod.POST)
    @RolePermission(permission = PermissionEnum.BOOK_MANAGER)
    public BeanResponse<Boolean> deleteRepalBook(@RequestBody DeleteRepalBookDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBookService.deleteRepalBook(dto));
    }

    @ApiOperation("用户查看账本列表和count")
    @RequestMapping(value = "/group", method = RequestMethod.GET)
    public BeanResponse<List<RepalBookInfoVO>> getRepalBookInfoList() {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBookService.getRepalBookInfoList());
    }

    @ApiOperation("设置首页金额可见性,应收/应付")
    @RequestMapping(value = "/visible/{type}", method = RequestMethod.POST)
    public BeanResponse<Boolean> setBookVisible(@PathVariable("type") Integer type) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBookService.setBookVisible(type));
    }

    @ApiOperation("新建账本")
    @RequestMapping(value = "/add", method = RequestMethod.POST)
    public BeanResponse<Long> addRepalBook(@RequestBody AddRepalBookDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBookService.addRepalBook(dto));
    }

    @ApiOperation("修改账本名称")
    @RequestMapping(value = "/name", method = RequestMethod.POST)
    public BeanResponse<Boolean> updateRepalBookName(@RequestBody RepalBookNameDTO dto) {
        return BeanResponse.success(CodeMsg.SUCCESS, repalBookService.updateRepalBookName(dto));
    }





//    @ApiOperation(value = "获取账本列表")
//    @RequestMapping( method = RequestMethod.GET)
//    public BeanResponse<List<AccountBookVO>> getAccountBookList() {
//        return BeanResponse.success(CodeMsg.SUCCESS, accountBookService.getAccountBookList(RequesterUtil.getCurrentUserId()));
//    }
//
//    @ApiOperation(value = "查看单个账本")
//    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
//    public BeanResponse<AccountBookDetailVO> getAccountBookById(@PathVariable("id") Long id) {
//        return BeanResponse.success(CodeMsg.SUCCESS, accountBookService.getAccountBookById(id, RequesterUtil.getCurrentUserId()));
//    }
//
//    @ApiOperation("设置账本name")
//    @RequestMapping(value = "/{id}", method = RequestMethod.PUT)
//    public BeanResponse<Boolean> updateAccountBookNameById(@PathVariable("id")Long id, @RequestBody UpdateAccountBookDTO dto) {
//        return BeanResponse.success(CodeMsg.SUCCESS, accountBookService.updateAccountBookNameById(dto.setUserId(RequesterUtil.getCurrentUserId()).setAccountBookId(id)));
//    }
//
//    @ApiOperation("刷新账本下所有账款评级")
//    @PostMapping("/refresh")
//    @Deprecated
//    public BeanResponse<Boolean> refreshGrade(@RequestBody RefreshGradeDTO refreshGradeDTO) {
//        return BeanResponse.success(CodeMsg.SUCCESS, accountBookService.refreshGrade(refreshGradeDTO,RequesterUtil.getCurrentUserId()));
//    }
//
//    @ApiOperation("删除账本")
//    @RequestMapping(value = "/{accountBookId}", method = RequestMethod.DELETE)
//    public BeanResponse<Boolean> deleteAccountBook(@PathVariable("accountBookId") Long accountBookId) {
//        return BeanResponse.success(CodeMsg.SUCCESS, accountBookService.deleteAccountBook(accountBookId, RequesterUtil.getCurrentUserId()));
//    }

}
