package com.winhc.repal.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.winhc.repal.constant.Constant;
import com.winhc.repal.entity.RepalRolePermission;
import com.winhc.repal.repository.RepalRolePermissionMapper;
import com.winhc.repal.service.RepalRolePermissionService;
import com.winhc.repal.util.RedisUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @description RepalRolePermission 接口实现类
 * @author Generator
 * @date 2022-04-08
 */
@Service
public class RepalRolePermissionServiceImpl extends ServiceImpl<RepalRolePermissionMapper, RepalRolePermission> implements RepalRolePermissionService {

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private RepalRolePermissionMapper repalRolePermissionMapper;


    @Override
    public Boolean checkUserPermission(Long userId, String code) {
        return getUserPermissionCodeSet(userId).contains(code);
    }

    @Override
    public Set<String> getUserPermissionCodeSet(Long userId) {
        Set<String> codeSet = redisUtil.setMembers(Constant.USER_PERMISSION_KEY + userId);
        if (CollUtil.isEmpty(codeSet)) {
            codeSet = repalRolePermissionMapper.getUserPermissionCodeSet(userId);
            if (CollUtil.isEmpty(codeSet)) {
                return new HashSet<>();
            }
            processUserPermissionCache(userId, codeSet);
        }
        return codeSet;
    }

    @Override
    public Boolean processUserPermissionCache(Long userId, Set<String> codeSet) {
        redisUtil.sAdd(Constant.USER_PERMISSION_KEY + userId, codeSet.toArray(new String[]{}));
        redisUtil.expire(Constant.USER_PERMISSION_KEY + userId, 7, TimeUnit.DAYS);
        return true;
    }
}
