package com.winhc.repal.util;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.winhc.repal.enums.DiagnosisTrendEnum;
import com.winhc.repal.model.vo.ResultItemVO;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description: DiagnosisUtil 评级工具类
 * @Author: xda
 * @Date: 2022/4/14 11:46
 */
public class DiagnosisUtil {

    /**
     *  评分趋势
     * @param preScore
     * @param nowScore
     * @param preGrade
     * @param nowGrade
     * @return
     */
    public static Integer getDiagnosisScoreTrend(BigDecimal preScore, BigDecimal nowScore, String preGrade, String nowGrade) {
        if (Objects.isNull(preScore) || StrUtil.isBlank(nowGrade)) {
            return DiagnosisTrendEnum.KEEP.getCode();
        } else {
            if (StrUtil.isNotBlank(preGrade)) {
                if (preScore.compareTo(nowScore) > 0) {
                    return DiagnosisTrendEnum.DESCENT.getCode();
                }
                if (preScore.compareTo(nowScore) < 0) {
                    return DiagnosisTrendEnum.ASCEND.getCode();
                }
            }
            return DiagnosisTrendEnum.KEEP.getCode();
        }
    }

    /**
     * 分数描述
     * @param score score
     * @return java.lang.String
     * @author xda
     * @date 2021/8/6 10:04
     */
    public static String getGradeConclusion(BigDecimal score) {
        if (Objects.isNull(score)) {
            return "";
        }
        if (score.compareTo(new BigDecimal("80")) >= 0) {
            return "低风险";
        } else if (score.compareTo(new BigDecimal("65")) >= 0) {
            return "较低风险";
        } else if (score.compareTo(new BigDecimal("55")) >= 0) {
            return "中风险";
        }
        return "高风险";
    }

    public static List<String> convertItemToList(String resultItem) {
        if (StrUtil.isBlank(resultItem)) {
            return new ArrayList<>();
        }
        List<ResultItemVO> itemList = JSON.parseArray(resultItem, ResultItemVO.class);
        if (CollUtil.isEmpty(itemList) || itemList.size() != 4) {
            return new ArrayList<>();
        }
        List<String> resultList = new ArrayList<>();
        Map<String, String> map = itemList.stream().collect(Collectors.toMap(ResultItemVO::getKey, ResultItemVO::getValue));
        resultList.add(map.get("资信情况"));
        resultList.add(map.get("履债能力"));
        resultList.add(map.get("追偿难度"));
        resultList.add(map.get("追偿周期"));
        return resultList.size() == 4 ? resultList : new ArrayList<>();
    }



}
